#define	 F_CPU	16000000UL
#include <util/delay.h>
#include <stdlib.h>
#include <stdbool.h>
#include "ili9341.h"
#include "ili9341gfx.h"

#define TEXT_SIZE 2
#define MENU_TEXT_SIZE 4
#define DELAY 2
#define OFFSET 4
#define NUM_PLAYERS 4
#define KEY_DELAY 300


#define PORTRAIT 0
#define LANDSCAPE 3
#define MENU NAVY

#define _0 48
#define _9 57
#define _A 65

extern uint16_t cursor_x, cursor_y,LCD_W,LCD_H;
static FILE mydata = FDEV_SETUP_STREAM(ili9341_putchar_printf, NULL, _FDEV_SETUP_WRITE);

typedef struct location {
	uint16_t x, y;
} location;

void init_LCD_and_ports();
char get_key();
void default_text_look();

void play_cricket();

void play_general(int p);
void play_301();
void play_501();

int main(void)
{
	init_LCD_and_ports();
	char key;
	
	ili9341_settextsize(MENU_TEXT_SIZE);
	_delay_ms(DELAY);
	ili9341_settextcolour(WHITE, MENU);
	_delay_ms(DELAY);
	
	printf(" Darts mode:\n\n");
	printf("- Cricket (A)\n\n");
	printf("- 301     (B)\n\n");
	printf("- 501     (C)");
	
	while(1) {
		key = get_key();
		
		if(key == 'A') {
			play_cricket();
		} else if(key == 'B') {
			play_general(301);
		} else if(key == 'C') {
			play_general(501);
		} else if(key == 'X') {
			continue;
		}
			
		_delay_ms(KEY_DELAY);
	}
	
	return 0;
}

void init_LCD_and_ports()
{
	// Start LCD
	stdout = & mydata;
	ili9341_init();//initial driver setup to drive ili9341
	_delay_ms(DELAY);
	ili9341_setRotation(LANDSCAPE); //rotate screen
	_delay_ms(DELAY);
	
	// Screen refresh
	ili9341_clear(MENU);
	
	// LED for input confirmation
	DDRD |= (1 << PD7);
	PORTD |= (1 << PD7);
	_delay_ms(DELAY);
		
	//Set column pins as input and row pins as output
	DDRA = 0xF0;
    _delay_ms(1);

	// Power up the row pins
    PORTA = 0x0F;
    _delay_ms(50);
}

char get_key() 
{
	uint8_t keypressed = 0;
		// If a button is pressed
	if (255 - PINA != 0b11110000) {
		_delay_ms(5);
		
		keypressed = PINA; //take the column value into integer
		
		DDRA ^= 0b11111111; //make rows as inputs and columns as ouput
		_delay_ms(1);

		PORTA ^= 0b11111111; //power up columns
		_delay_ms(1);
		
		keypressed |= PINA; // take row value and OR it to column value
		keypressed = 255 - keypressed;
		
		if (keypressed == 0b00010001) { // 1
			PORTD ^= (1 << PD7);
			return '1';
		}

		if (keypressed == 0b00010010) { // 4
			PORTD ^= (1 << PD7);
			return '4';
		}

		if (keypressed == 0b00010100) { // 7
			PORTD ^= (1 << PD7);
			return '7';
		}

		if (keypressed == 0b00011000) { // *
			PORTD ^= (1 << PD7);
			return '*';
		}


		if (keypressed == 0b00100001) { // 2
			PORTD ^= (1 << PD7);
			return '2';
		}

		if (keypressed == 0b00100010) { // 5
			PORTD ^= (1 << PD7);
			return '5';
		}

		if (keypressed == 0b00100100) { // 8
			PORTD ^= (1 << PD7);
			return '8';
		}

		if (keypressed == 0b00101000) { // 0
			PORTD ^= (1 << PD7);
			return '0';
		}


		if (keypressed == 0b01000001) { // 3
			PORTD ^= (1 << PD7);
			return '3';
		}

		if (keypressed == 0b01000010) { // 6
			PORTD ^= (1 << PD7);
			return '6';
		}

		if (keypressed == 0b01000100) { // 9
			PORTD ^= (1 << PD7);
			return '9';
		}

		if (keypressed == 0b01001000) { // #
			PORTD ^= (1 << PD7);
			return '#';
		}

		
		if (keypressed == 0b10000001) { // A
			PORTD ^= (1 << PD7);
			return 'A';
		}

		if (keypressed == 0b10000010) { // B
			PORTD ^= (1 << PD7);
			return 'B';
		}

		if (keypressed == 0b10000100) { // C
			PORTD ^= (1 << PD7);
			return 'C';
		}

		if (keypressed == 0b10001000) { // D
			PORTD ^= (1 << PD7);
			return 'D';
		}
		
		keypressed = 0; //after showing integer, erase the row column memory

		DDRA ^= 0b11111111; //shift input and power port
		_delay_ms(1);

		PORTA ^= 0b11111111; //power up row pins of keypad

	}
	return 'X';
}

void default_text_look() 
{
	ili9341_settextcolour(WHITE,BLACK);
	_delay_ms(DELAY);
	ili9341_settextsize(TEXT_SIZE);
	_delay_ms(DELAY);
	ili9341_setcursor(OFFSET,OFFSET);	
	_delay_ms(DELAY);
}

void play_cricket()
{
	ili9341_clear(BLACK);
	default_text_look();
	uint8_t current_player = 0;
	uint16_t i;
	// Color for each player
	uint16_t player_colors[2] = {RED, GREEN};
	// Identifier for each player
	char player_character[2] = {'A', 'B'};
	
	uint8_t hit[2][7] = {{0}};
	location hit_x[2][7];
	
	// Position of player names
	location players_offset[2];
	// PlayerA position
	players_offset[0].x = OFFSET;
	players_offset[0].y = OFFSET;
	// PlayerB position
	players_offset[1].x = OFFSET;
	players_offset[1].y = LCD_H - 1 - OFFSET - 8 * TEXT_SIZE;
	
	for(i = 0; i < 2; i++) {
		// Use preffered color for specific player
		ili9341_settextcolour(player_colors[i],BLACK);
		ili9341_setcursor(players_offset[i].x, players_offset[i].y);	
		printf("          Player%c", player_character[i]);
	}
	
	// Draw labels
	ili9341_settextcolour(YELLOW,BLACK);
	ili9341_drawvline(0, LCD_H / 2 - 65, 130, DARKGREY);
	
	for(i = 0; i < 6; i++) {
		if(i == 0) {
			ili9341_setcursor(OFFSET + 45 * i + 10, (LCD_H / 2) - 6);

			hit_x[0][0].x = OFFSET + 45 * i + 8 * TEXT_SIZE;
			hit_x[0][0].y = (LCD_H / 2) - 36;
			hit_x[1][0].x = OFFSET + 45 * i + 8 * TEXT_SIZE;
			hit_x[1][0].y = (LCD_H / 2) + 20;
		} else {
			ili9341_setcursor(OFFSET + 45 * i + 12, (LCD_H / 2) - 6);
			
			hit_x[0][i].x = OFFSET + 45 * i + 9 * TEXT_SIZE;
			hit_x[0][i].y = (LCD_H / 2) - 36;
			hit_x[1][i].x = OFFSET + 45 * i + 9 * TEXT_SIZE;
			hit_x[1][i].y = (LCD_H / 2) + 20;
		}
		_delay_ms(DELAY);
		
		printf("%d", 20 - i);
		ili9341_drawvline(OFFSET + 45 * (i + 1), LCD_H / 2 - 65, 130, DARKGREY);
	}
	ili9341_setcursor(OFFSET + 45 * i + 17, (LCD_H / 2) - 6);	
	_delay_ms(DELAY);

	hit_x[0][i].x = OFFSET + 45 * i + 9 * TEXT_SIZE;
	hit_x[0][i].y = (LCD_H / 2) - 36;
	hit_x[1][i].x = OFFSET + 45 * i + 9 * TEXT_SIZE;
	hit_x[1][i].y = (LCD_H / 2) + 20;
	
	printf("B");
	
	ili9341_drawvline(OFFSET + 45 * (i + 1), LCD_H / 2 - 65, 130, DARKGREY);
	ili9341_settextcolour(player_colors[i],BLACK);
	_delay_ms(DELAY);
	
	ili9341_drawhline(0, LCD_H / 2 + 8 * TEXT_SIZE, LCD_W, DARKGREY);
	ili9341_drawhline(0, LCD_H / 2 - 8 * TEXT_SIZE, LCD_W, DARKGREY);
	
	char key;
	
	ili9341_settextcolour(YELLOW,BLACK);
	_delay_ms(DELAY);
	
	ili9341_setcursor(players_offset[current_player].x, players_offset[current_player].y);	
	printf("      -->");
	
	while(1) {
		key = get_key();
		
		if(key == 'X') {
			continue;
		} else if(key == 'A') {
			ili9341_setcursor(players_offset[current_player].x, players_offset[current_player].y);	
			printf("         ");
			current_player = 0;
			ili9341_setcursor(players_offset[current_player].x, players_offset[current_player].y);	
			printf("      -->");
		} else if(key == 'B') {
			ili9341_setcursor(players_offset[current_player].x, players_offset[current_player].y);	
			printf("         ");
			current_player = 1;
			ili9341_setcursor(players_offset[current_player].x, players_offset[current_player].y);	
			printf("      -->");
		} else if(key == '1' && hit[current_player][0] < 3) {
			if(current_player == 0)
				ili9341_setcursor(hit_x[current_player][0].x, hit_x[current_player][0].y - 8 * TEXT_SIZE * hit[current_player][0]);
			else
				ili9341_setcursor(hit_x[current_player][0].x, hit_x[current_player][0].y + 8 * TEXT_SIZE * hit[current_player][0]);
			printf("x");
			hit[current_player][0]++;
		} else if(key == '2' && hit[current_player][1] < 3) {
			if(current_player == 0)
				ili9341_setcursor(hit_x[current_player][1].x, hit_x[current_player][1].y - 8 * TEXT_SIZE * hit[current_player][1]);
			else
				ili9341_setcursor(hit_x[current_player][1].x, hit_x[current_player][1].y + 8 * TEXT_SIZE * hit[current_player][1]);
			printf("x");
			hit[current_player][1]++;
		} else if(key == '3' && hit[current_player][2] < 3) {
			if(current_player == 0)
				ili9341_setcursor(hit_x[current_player][2].x, hit_x[current_player][2].y - 8 * TEXT_SIZE * hit[current_player][2]);
			else
				ili9341_setcursor(hit_x[current_player][2].x, hit_x[current_player][2].y + 8 * TEXT_SIZE * hit[current_player][2]);
			printf("x");
			hit[current_player][2]++;
		} else if(key == '4' && hit[current_player][3] < 3) {
			if(current_player == 0)
				ili9341_setcursor(hit_x[current_player][3].x, hit_x[current_player][3].y - 8 * TEXT_SIZE * hit[current_player][3]);
			else
				ili9341_setcursor(hit_x[current_player][3].x, hit_x[current_player][3].y + 8 * TEXT_SIZE * hit[current_player][3]);
			printf("x");
			hit[current_player][3]++;
		} else if(key == '5' && hit[current_player][4] < 3) {
			if(current_player == 0)
				ili9341_setcursor(hit_x[current_player][4].x, hit_x[current_player][4].y - 8 * TEXT_SIZE * hit[current_player][4]);
			else
				ili9341_setcursor(hit_x[current_player][4].x, hit_x[current_player][4].y + 8 * TEXT_SIZE * hit[current_player][4]);
			printf("x");
			hit[current_player][4]++;
		} else if(key == '6' && hit[current_player][5] < 3) {
			if(current_player == 0)
				ili9341_setcursor(hit_x[current_player][5].x, hit_x[current_player][5].y - 8 * TEXT_SIZE * hit[current_player][5]);
			else
				ili9341_setcursor(hit_x[current_player][5].x, hit_x[current_player][5].y + 8 * TEXT_SIZE * hit[current_player][5]);
			printf("x");
			hit[current_player][5]++;
		}
		 else if(key == '7' && hit[current_player][6] < 3) {
			if(current_player == 0)
				ili9341_setcursor(hit_x[current_player][6].x, hit_x[current_player][6].y - 8 * TEXT_SIZE * hit[current_player][6]);
			else
				ili9341_setcursor(hit_x[current_player][6].x, hit_x[current_player][6].y + 8 * TEXT_SIZE * hit[current_player][6]);
			printf("x");
			hit[current_player][6]++;
		}
	
		_delay_ms(KEY_DELAY);
	}
}

void play_general(int p)
{
	ili9341_clear(BLACK);
	default_text_look();
	
	ili9341_drawvline(LCD_W / 2, 0, LCD_H, DARKGREY);
	ili9341_drawhline(0, LCD_H / 2, LCD_W, DARKGREY);
	
	uint16_t i;
	char key;
	uint8_t current_player = 0;
	ili9341_drawvline(LCD_W / 2, 0, 	    LCD_H / 2, YELLOW);
	ili9341_drawvline(0, 		 0, 		LCD_H / 2, YELLOW);
	ili9341_drawhline(0, 		 LCD_H / 2, LCD_W / 2, YELLOW);
	ili9341_drawhline(0, 		 0, 		LCD_W / 2, YELLOW);
	
	// Color for each player
	int player_colors[4] = {RED, GREEN, BLUE, PURPLE};
	// Identifier for each player
	char player_character[4] = {'A', 'B', 'C', 'D'};
	// Number of wins for each player
	int wins[4] = {0, 0, 0, 0};
	
	// Array of digits
	uint8_t digit[4] = {0, 0, 0, 0};
	// Array of scores
	char scores[4][3] = {{'\0'}};
	// Array of positions for score_offsets
	location score_offsets[4];
	// Player0 score position
	score_offsets[0].x = OFFSET;
	score_offsets[0].y = OFFSET + (8 * TEXT_SIZE * 6);
	// Player1 score position
	score_offsets[1].x = OFFSET + (LCD_W / 2);
	score_offsets[1].y = OFFSET + (8 * TEXT_SIZE * 6);
	// Player2 score position
	score_offsets[2].x = OFFSET;
	score_offsets[2].y = OFFSET + (8 * TEXT_SIZE * 6) + (LCD_H / 2);
	// Player3 score position
	score_offsets[3].x = OFFSET + (LCD_W / 2);
	score_offsets[3].y = OFFSET + (8 * TEXT_SIZE * 6) + (LCD_H / 2);
	
	for(i = 0; i < NUM_PLAYERS; i++) {
		ili9341_setcursor(score_offsets[i].x, score_offsets[i].y);	
		_delay_ms(DELAY);
		printf("Scored: ");
		
		//Set a new offset starting after the ": "
		score_offsets[i].x = cursor_x;
		score_offsets[i].y = cursor_y;
	}
	
	// Array of starting points for players
	int16_t points[4] = {p, p, p, p};
	// Array of positions for current points
	location points_offset[4];
	// Array of positions for players
	location players_offset[4];
	// PlayerA position
	players_offset[0].x = OFFSET;
	players_offset[0].y = OFFSET;
	// PlayerB position
	players_offset[1].x = OFFSET + (LCD_W / 2);
	players_offset[1].y = OFFSET;
	// PlayerC position
	players_offset[2].x = OFFSET;
	players_offset[2].y = OFFSET + (LCD_H / 2);
	// PlayerD position
	players_offset[3].x = OFFSET + (LCD_W / 2);
	players_offset[3].y = OFFSET + (LCD_H / 2);
	
	for(i = 0; i < NUM_PLAYERS; i++) {
		// Use preffered color for specific player
		ili9341_settextcolour(player_colors[i],BLACK);
		_delay_ms(DELAY);
		ili9341_setcursor(players_offset[i].x, players_offset[i].y);	
		_delay_ms(DELAY);
		printf("   Player%c", player_character[i]);
		
		// Set new permanent coors for points number
		points_offset[i].x = cursor_x - 85;
		points_offset[i].y = cursor_y + (8 * TEXT_SIZE * 3);
		
		// Use increased size for points
		ili9341_settextsize(TEXT_SIZE + 3);
		_delay_ms(DELAY);
		ili9341_setcursor(points_offset[i].x, points_offset[i].y);	
		_delay_ms(DELAY);
		printf("%d", points[i]);
		
		// Set size back to original
		ili9341_settextsize(TEXT_SIZE);
		_delay_ms(DELAY);
	}
	
	// Restore color of text
	ili9341_settextcolour(WHITE,BLACK);
	_delay_ms(DELAY);

	while(1) {
		key = get_key();
		
		if(key == 'X') { // dummy ret value
			continue;
		} else if(key >= _0 && key <= _9){ // entered a digit
			// override last digit if there are 3 already
			if(digit[current_player] == 3) {
				digit[current_player]--;
			}
			scores[current_player][digit[current_player]] = key;
			
			ili9341_setcursor(score_offsets[current_player].x + 8 * TEXT_SIZE * digit[current_player], 
						      score_offsets[current_player].y);	
			_delay_ms(DELAY);
			printf("%c", key);
			
			digit[current_player]++;
		} else if(key == '#') { // delete digit
			if(digit[current_player] > 0) {
				digit[current_player]--;
				scores[current_player][digit[current_player]] = '\0';
				
				ili9341_setcursor(score_offsets[current_player].x + 8 * TEXT_SIZE * digit[current_player], 
								  score_offsets[current_player].y);		
				_delay_ms(DELAY);
				printf(" ");
			}
		} else if(key == '*' && digit[current_player] > 0) { // calculate score
			int16_t temp_score = atoi(scores[current_player]);
			// You must reach exactly 0, cannot go below
			if(points[current_player] - temp_score >= 0) {
				points[current_player] -= temp_score;
			
				// Use increased size for points
				ili9341_settextsize(TEXT_SIZE + 3);
				_delay_ms(DELAY);
				ili9341_settextcolour(player_colors[current_player],BLACK);
				_delay_ms(DELAY);
				
				// Some formatting to keep points in center
				if(points[current_player] > 9 && points[current_player] < 100) {
					ili9341_setcursor(points_offset[current_player].x - 20, points_offset[current_player].y);	
				} else {
					ili9341_setcursor(points_offset[current_player].x, points_offset[current_player].y);	
				}
				_delay_ms(DELAY);
				if(points[current_player] < 100) {
					printf(" ");
				}
				printf("%d  ", points[current_player]);
				
				// Set back the size and color
				ili9341_settextcolour(WHITE,BLACK);
				_delay_ms(DELAY);
				ili9341_settextsize(TEXT_SIZE);
				_delay_ms(DELAY);
				
				// Redraw the vertical line on the right
				switch(current_player) {
					case 0:
						ili9341_drawvline(LCD_W / 2, 0, 		LCD_H / 2, YELLOW);
						break;
						
					case 1:
						ili9341_drawvline(LCD_W - 1, 0, 		LCD_H / 2, YELLOW);
						break;
						
					case 2:
						ili9341_drawvline(LCD_W / 2, LCD_H / 2, LCD_H / 2, YELLOW);
						break;
						
					case 3:
						ili9341_drawvline(LCD_W - 1, LCD_H / 2, LCD_H / 2, YELLOW);
						break;
				}
				
				// Declare winner if any reaches 0 points
				if(points[current_player] == 0) {
					
					_delay_ms(1000);
					for(i = 0; i < NUM_PLAYERS; i++) {
						points[i] = p;
						// Use preffered color for specific player
						ili9341_settextcolour(player_colors[i],BLACK);
						_delay_ms(DELAY);
						ili9341_setcursor(players_offset[i].x, players_offset[i].y);	
						
						// Use increased size for points
						ili9341_settextsize(TEXT_SIZE + 3);
						_delay_ms(DELAY);
						ili9341_setcursor(points_offset[i].x, points_offset[i].y);	
						_delay_ms(DELAY);
						printf("%d", points[i]);
					}
					
					// Restore color of text
					ili9341_settextcolour(WHITE,BLACK);
					_delay_ms(DELAY);
					// Set a small size
					ili9341_settextsize(TEXT_SIZE - 1);
					_delay_ms(DELAY);
					
					ili9341_setcursor(points_offset[current_player].x, 
									  points_offset[current_player].y - 8 * TEXT_SIZE * 2);	
					wins[current_player] += 1;
					printf("   wins: %d", wins[current_player]);
					
					// Set size back to original
					ili9341_settextsize(TEXT_SIZE);
					_delay_ms(DELAY);
				}
			}
			
			// Clean up score
			ili9341_setcursor(score_offsets[current_player].x, score_offsets[current_player].y);	
			_delay_ms(DELAY);
			printf("    ");
			digit[current_player] = 0;
			for(i = 0; i < 3; i++)
				scores[current_player][i] = '\0';
			// Change current player and draw a yellow box to show it
		} else if(key == 'A' || key == 'B' || key == 'C' || key == 'D') {
			ili9341_drawvline(LCD_W / 2, 0, 		LCD_H, DARKGREY);
			ili9341_drawhline(0, 		 LCD_H / 2, LCD_W, DARKGREY);
			
			ili9341_drawvline(0, 		 0, 		LCD_H, BLACK);
			ili9341_drawhline(0, 		 0, 		LCD_W, BLACK);
			ili9341_drawvline(LCD_W - 1, 0, 		LCD_H, BLACK);
			ili9341_drawhline(0, 		 LCD_H - 1, LCD_W, BLACK);
			switch(key) {
				case 'A':
					current_player = 0;
					ili9341_drawvline(LCD_W / 2, 0, 	    LCD_H / 2, YELLOW);
					ili9341_drawvline(0, 		 0, 		LCD_H / 2, YELLOW);
					ili9341_drawhline(0, 		 LCD_H / 2, LCD_W / 2, YELLOW);
					ili9341_drawhline(0, 		 0, 		LCD_W / 2, YELLOW);
					break;
				case 'B':
					current_player = 1;
					ili9341_drawvline(LCD_W - 1, 0, 		LCD_H / 2, YELLOW);
					ili9341_drawvline(LCD_W / 2, 0, 		LCD_H / 2, YELLOW);
					ili9341_drawhline(LCD_W / 2, LCD_H / 2, LCD_W / 2, YELLOW);
					ili9341_drawhline(LCD_W / 2, 0, 		LCD_W / 2, YELLOW);
					break;
				case 'C':
					current_player = 2;
					ili9341_drawvline(LCD_W / 2, LCD_H / 2,	LCD_H / 2, YELLOW);
					ili9341_drawvline(0, 		 LCD_H / 2,	LCD_H / 2, YELLOW);
					ili9341_drawhline(0, 		 LCD_H / 2, LCD_W / 2, YELLOW);
					ili9341_drawhline(0, 		 LCD_H - 1,	LCD_W / 2, YELLOW);
					break;
				case 'D':
					current_player = 3;
					ili9341_drawvline(LCD_W - 1, LCD_H / 2, LCD_H / 2, YELLOW);
					ili9341_drawvline(LCD_W / 2, LCD_H / 2, LCD_H / 2, YELLOW);
					ili9341_drawhline(LCD_W / 2, LCD_H / 2, LCD_W / 2, YELLOW);
					ili9341_drawhline(LCD_W / 2, LCD_H - 1, LCD_W - 1, YELLOW);
					break;
			}
		}
		_delay_ms(KEY_DELAY);
	}
}
